#ifndef BASE_DATA_STRUCT_H
#define BASE_DATA_STRUCT_H

#include <memory>
#include <unordered_map>

namespace algorithm{
	namespace base{
		// վݽṹ
		struct ReceiveData{
			unsigned int reader_id;		// վ
			unsigned short antenna_id;  // ߺ
			long long rec_time_stamp;	// վʱ䣬һ7ֽڵ޷
			int special;
			double x;	//վx
			double y;   //վy
			double z;   //վz

			ReceiveData(){
				reader_id = 0;
				antenna_id = 0;
				rec_time_stamp = 0;
				x = y = z = 0.0;
				special = 0;
			};
		};

		typedef unordered_map<unsigned long long ,std::shared_ptr<ReceiveData>> ReceiveDataUnorderedMap;

		template<typename _Scalar, int NX = Eigen::Dynamic, int NY = Eigen::Dynamic>
    class Functor
    {
    private:
        int m_inputs, m_values;
    public:
        typedef _Scalar Scalar;
        enum {
            InputsAtCompileTime = NX,
            ValuesAtCompileTime = NY
        };
        typedef Eigen::Matrix<Scalar,InputsAtCompileTime,1> InputType;
        typedef Eigen::Matrix<Scalar,ValuesAtCompileTime,1> ValueType;
        typedef Eigen::Matrix<Scalar,ValuesAtCompileTime,InputsAtCompileTime> JacobianType;

        Functor() : m_inputs(InputsAtCompileTime), m_values(ValuesAtCompileTime) {}
        Functor(int inputs, int values) : m_inputs(inputs), m_values(values) {}

        int inputs() const { return m_inputs; }
        int values() const { return m_values; }

    };

    class TDOAFunctor : public Functor<double>
    {
    private:
        Eigen::MatrixXd _data, _pos;
    public:
        TDOAFunctor(const Eigen::MatrixXd &data,const Eigen::MatrixXd &pos): Functor<double>(3,3),_data(data),_pos(pos) {}
        int operator()(const Eigen::VectorXd &x, Eigen::VectorXd &fvec) const
        {
            for (int i = 0; i < values(); i++)
            {
                fvec(i) = sqrt(pow(_data(i,0) - x(0),2) + pow(_data(i,1) - x(1),2) + pow(_data(i,2) - x(2),2)) 
                        - sqrt(pow(_pos(0) - x(0),2) + pow(_pos(1) - x(1),2) + pow(_pos(2) - x(2),2)) 
                        - _data(i,3);
            }
            return 0;
        }
    };
	};

	const int MAX_READER_TDOA_PATH_NUMS = 10;	//ҪΪ䴦ܴڶԭ
	const double INVALID_COORDINATE = -1000.0;

	enum DIMENSION{
		_1D = 1,
		_2D,
		_3D
	};
	enum ALGO_LOC_TYPE{
		ALGO_LOC_SUCCESSED = 0,			//㷨λɹ
		ALGO_FAILED_CONDITION_1,		//ͬ5
		ALGO_FAILED_CONDITION_2,		//ͬкţʱͬȴ5
		ALGO_FAILED_CONDITION_3,		//ʱֵͬ쳣
		ALGO_FAILED_CONDITION_4,		//ϴʱڿıʱ
		ALGO_FAILED_CONDITION_5,		//˿ٶȳ
		ALGO_FAILED_CONDITION_6,		//ٶȳ
		ALGO_FAILED_CONDITION_7,		//ٶȳ
		ALGO_FAILED_CONDITION_8,		//2sλʧܵ
		ALGO_FAILED_CONDITION_9,		//վʱľֵڷվ֮ľ
		ALGO_FAILED_CONDITION_10,		//վ֮޵ͼ
		ALGO_FAILED_CONDITION_11,		//λ޽orĸΪ0
		ALGO_FAILED_CONDITION_12,		//վС4mվվ
		ALGO_FAILED_CONDITION_13,		//޵ͼվ֮ľ10m
		ALGO_FAILED_CONDITION_14,		//λ붨λվ֮ľ4
		ALGO_FAILED_CONDITION_15,		//붨λС2
		ALGO_LOC_TOTAL
	};

	struct POS{
		double posx; // Ϊλ
		double posy;
		double posz;
		//Χ
		double pos_radius;

		//Ϊλ꣬/ͼ = 
		double cx;
		double cy;
		double cz;
		//ٶ
		double cvx;
		double cvy;
		double cvz;
		//ٶ
		double av;

		//涨λվϢ
		int nFirstReader;
		int nSecondReader;

		//Ȳο
		double dDiff[MAX_READER_TDOA_PATH_NUMS];
		double dis_diff;

		//λʧܵ״̬
		int reason;
		int status;
		int dimension;
		unsigned long long card_count;		//ct
		unsigned short card_timestamp; 
		bool update;		//ǷҪ浽ͬ
		bool is_back;
		bool is_fit;		//Ƿ

		//ǰվͬοݵķվͬʱֵ
		double diff_reader_sync_num;

		double delta_time;			//ʱ
		//οݵĶλ
		double ref_x;
		double ref_y;
		double ref_z;
		double origin_speed;	//ԭʼλٶ

		double sumVariance;		// λֵ֮ĺ

		POS(){
			card_timestamp = 0;
			nFirstReader = 0;
			nSecondReader = 0;
			pos_radius = 999999.9;	
			posx = posy = posz = INVALID_COORDINATE;
			cx = cy = cz = cvx = cvy = cvz = 0.0;
			av = 0.0;
			status = 0;
			card_count = 0;
			for (int i=0;i<MAX_READER_TDOA_PATH_NUMS;i++)
			{
				dDiff[i] = 0;
			}

			dis_diff = 999999.9;
			update = false;
			is_fit = false;
			is_back = false;
			origin_speed = 0;
			sumVariance = 0;
			diff_reader_sync_num = 0;
			delta_time = 0.0;
			ref_x = ref_y = ref_z = 0.0;
			dimension = _1D;
			reason = ALGO_LOC_SUCCESSED;
		}

		POS& operator=(POS& tmp){
			this->card_count = tmp.card_count;
			this->card_timestamp = tmp.card_timestamp;
			nFirstReader = tmp.nFirstReader;
			nSecondReader = tmp.nSecondReader;
			posx = tmp.posx;
			posy = tmp.posy;
			posz = tmp.posz;
			cx = tmp.cx;
			cy = tmp.cy;
			cz = tmp.cz;
			cvx = tmp.cvx;
			cvy = tmp.cvy;
			cvz = tmp.cvz;
			av = tmp.av;
			update = tmp.update;
			origin_speed = tmp.origin_speed;
			status = tmp.status;
			is_back = tmp.is_back;
			diff_reader_sync_num = tmp.diff_reader_sync_num;
			delta_time = tmp.delta_time;
			dis_diff = tmp.dis_diff;
			ref_x = tmp.ref_x;
			ref_y = tmp.ref_y;
			ref_z = tmp.ref_z;
			sumVariance = tmp.sumVariance;
			for (int i=0;i<MAX_READER_TDOA_PATH_NUMS;i++)
			{
				dDiff[i] = tmp.dDiff[i];
			}
			pos_radius = tmp.pos_radius;
			reason = ALGO_LOC_SUCCESSED;
			dimension = tmp.dimension;

			return *this;
		}
	};

	// λ
	struct _coordinate{
		_coordinate(){
			t = 0;
			reader_id = 0;
			tt = 0;
			d = 0.0;
			sync_num = 0;	

			x = 0.0;
			y = 0.0;
			z = 0.0;
			a = 0.0;
			v = 0.0;
			antenna_id = 0;
			d_offset = 0.0;
			special = 0;
			acceleration = 0;
			acce_state = 0;	
			acce_state_last = 0;
			ins_direction = 0;
			rangingtype = 1;		// 0 tof1 toda
			cur_time = time(NULL);
			ins_request_check = 0;
			dimension = _1D;
			strength = 0;
		}
		int t; // λʱ
		int reader_id; // վ
		unsigned long long syncRootId; // tdoaڵ
		unsigned long long tt; // վյʱ䣬ΪͬԲֵʱ
		double d; // 

		double x; // x
		double y; // y
		double z; // z
		double a; // ƽǶ
		double v; // ٶ
		int antenna_id;

		int sync_num;			//ͬ
		int acce_state;			//ٶ״̬
		int acce_state_last;	//ٶȼһ״̬
		int ins_direction;		//ߵ
		int special;			//վǷվ0վ1ͨվ
		int rangingtype;
		int reason;
		int ins_request_check;
		int dimension;		//ָһάάά
		int strength;

		double acceleration;	//ٶ
		double d_offset;		// ʾƫƣվ
		double recv_signal_power; //źŹʣֵ

		time_t cur_time;
		_coordinate& operator=(_coordinate &tmp){
			t = tmp.t;
			reader_id = tmp.reader_id;
			tt = tmp.tt;
			d = tmp.d;
			sync_num = tmp.sync_num;
			x = tmp.x;
			y = tmp.y;
			z = tmp.z;
			a = tmp.a;
			v = tmp.v;
			antenna_id = tmp.antenna_id;
			d_offset = tmp.d_offset;
			special = tmp.special;
			acceleration = tmp.acceleration;
			acce_state = tmp.acce_state;
			acce_state_last = tmp.acce_state_last;
			ins_direction = tmp.ins_direction;
			rangingtype = tmp.rangingtype;
			reason = ALGO_LOC_SUCCESSED;
			cur_time = tmp.cur_time;
			ins_request_check = tmp.ins_request_check;
			dimension = tmp.dimension;
			strength = tmp.strength;

			return *this;
		}
		bool operator==(_coordinate &tmp)
		{
			if (x == tmp.x && y == tmp.y)
			{
				return true;
			}
			return false;
		}
	};
};

#endif